import unittest
import os
import shutil
import json
import time
import requests_mock
import tempfile
import logging
from unittest.mock import patch, MagicMock, mock_open
import urllib.request

from iadrive.core import IAdrive
from iadrive import __version__


SCANNER = f'IAdrive File Mirroring Application {__version__}'

current_path = os.path.dirname(os.path.realpath(__file__))


def get_testfile_path(name):
    return os.path.join(current_path, 'test_iadrive_files', name)


class MockGdown:
    @staticmethod
    def download(url, output=None, quiet=True, fuzzy=True):
        """Mock gdown download for single files"""
        if not os.path.exists(output):
            os.makedirs(output, exist_ok=True)
        
        # Create a dummy file
        test_file = os.path.join(output, 'test_file.txt')
        with open(test_file, 'w') as f:
            f.write('Mock downloaded content')
        return test_file
    
    @staticmethod
    def download_folder(url, output=None, quiet=True):
        """Mock gdown download_folder for folders"""
        os.makedirs(output, exist_ok=True)
        
        # Create a folder structure
        folder_name = 'Test Folder'
        folder_path = os.path.join(output, folder_name)
        os.makedirs(folder_path, exist_ok=True)
        
        # Create some test files
        files = ['file1.txt', 'file2.pdf', 'subfolder/file3.docx']
        for file_path in files:
            full_path = os.path.join(folder_path, file_path)
            os.makedirs(os.path.dirname(full_path), exist_ok=True)
            with open(full_path, 'w') as f:
                f.write(f'Mock content for {file_path}')


class MockUrllibRequest:
    """Mock for urllib.request to simulate Google Docs downloads"""
    
    @staticmethod
    def urlretrieve(url, filename):
        """Mock urlretrieve for Google Docs export"""
        # Create directory if it doesn't exist
        os.makedirs(os.path.dirname(filename), exist_ok=True)
        
        # Create mock file content based on format
        ext = os.path.splitext(filename)[1].lower()
        if ext == '.pdf':
            content = b'%PDF-1.4 Mock PDF content'
        elif ext == '.docx':
            content = b'PK\x03\x04 Mock DOCX content'  # DOCX files start with PK
        elif ext == '.txt':
            content = b'Mock text content from Google Doc'
        elif ext == '.html':
            content = b'<html><body>Mock HTML content</body></html>'
        elif ext == '.csv':
            content = b'Column1,Column2,Column3\nValue1,Value2,Value3'
        else:
            content = f'Mock {ext} content'.encode('utf-8')
        
        with open(filename, 'wb') as f:
            f.write(content)
    
    @staticmethod
    def urlopen(url, timeout=None):
        """Mock urlopen for getting document titles"""
        class MockResponse:
            def read(self):
                if 'document' in url:
                    return b'<html><head><title>Test Document - Google Docs</title></head></html>'
                elif 'spreadsheets' in url:
                    return b'<html><head><title>Test Spreadsheet - Google Sheets</title></head></html>'
                elif 'presentation' in url:
                    return b'<html><head><title>Test Presentation - Google Slides</title></head></html>'
                return b'<html><head><title>Untitled Document</title></head></html>'
            
            def __enter__(self):
                return self
            
            def __exit__(self, *args):
                pass
        
        return MockResponse()


class IAdriveMockTests(unittest.TestCase):

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        self.iadrive = IAdrive(verbose=False, dir_path=self.test_dir)
        self.maxDiff = None

    def tearDown(self):
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    def test_set_dir_path(self):
        """Test that directory is created correctly"""
        test_path = os.path.join(self.test_dir, 'custom_dir')
        iadrive = IAdrive(dir_path=test_path)
        
        self.assertTrue(os.path.exists(test_path))
        self.assertEqual(iadrive.dir_path, test_path)

    def test_logger_quiet_mode(self):
        """Test logger in quiet mode"""
        self.assertIsInstance(self.iadrive.logger, logging.Logger)
        self.assertEqual(self.iadrive.logger.level, logging.ERROR)

    def test_logger_verbose_mode(self):
        """Test logger in verbose mode"""
        iadrive = IAdrive(verbose=True, dir_path=self.test_dir)
        self.assertIsInstance(iadrive.logger, logging.Logger)

    # Original Google Drive tests
    def test_extract_drive_id_folder(self):
        """Test extracting drive ID from folder URL"""
        url = 'https://drive.google.com/drive/folders/1-0axLqCuOUNbBIe3Cz6Y1KojGg4iXg1h'
        result = self.iadrive.extract_drive_id(url)
        expected = '1-0axLqCuOUNbBIe3Cz6Y1KojGg4iXg1h'
        self.assertEqual(result, expected)

    def test_extract_drive_id_file(self):
        """Test extracting drive ID from file URL"""
        url = 'https://drive.google.com/file/d/1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms/edit'
        result = self.iadrive.extract_drive_id(url)
        expected = '1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms'
        self.assertEqual(result, expected)

    def test_is_folder_url(self):
        """Test folder URL detection"""
        folder_url = 'https://drive.google.com/drive/folders/1-0axLqCuOUNbBIe3Cz6Y1KojGg4iXg1h'
        file_url = 'https://drive.google.com/file/d/1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms/edit'
        
        self.assertTrue(self.iadrive.is_folder_url(folder_url))
        self.assertFalse(self.iadrive.is_folder_url(file_url))

    # Google Docs specific tests
    def test_is_google_docs_url(self):
        """Test Google Docs URL detection"""
        docs_urls = [
            'https://docs.google.com/document/d/1abc123/edit',
            'https://docs.google.com/spreadsheets/d/1abc123/edit',
            'https://docs.google.com/presentation/d/1abc123/edit'
        ]
        drive_url = 'https://drive.google.com/file/d/1abc123'
        
        for url in docs_urls:
            self.assertTrue(self.iadrive.is_google_docs_url(url))
        
        self.assertFalse(self.iadrive.is_google_docs_url(drive_url))

    def test_get_google_docs_type(self):
        """Test getting Google Docs type"""
        test_cases = [
            ('https://docs.google.com/document/d/1abc123/edit', 'document'),
            ('https://docs.google.com/spreadsheets/d/1abc123/edit', 'spreadsheets'),
            ('https://docs.google.com/presentation/d/1abc123/edit', 'presentation'),
            ('https://drive.google.com/file/d/1abc123', None)
        ]
        
        for url, expected in test_cases:
            result = self.iadrive.get_google_docs_type(url)
            self.assertEqual(result, expected)

    def test_extract_docs_id(self):
        """Test extracting Google Docs ID"""
        test_cases = [
            ('https://docs.google.com/document/d/1abc123def456/edit', '1abc123def456'),
            ('https://docs.google.com/spreadsheets/d/1xyz789/edit#gid=0', '1xyz789'),
            ('https://docs.google.com/presentation/d/1pqr456/edit#slide=1', '1pqr456')
        ]
        
        for url, expected in test_cases:
            result = self.iadrive.extract_docs_id(url)
            self.assertEqual(result, expected)

    def test_extract_docs_id_invalid_url(self):
        """Test extracting docs ID from invalid URL"""
        url = 'https://example.com/invalid'
        with self.assertRaises(ValueError):
            self.iadrive.extract_docs_id(url)

    @patch('iadrive.core.urllib.request', MockUrllibRequest)
    def test_download_google_doc_all_formats(self):
        """Test downloading Google Doc with all formats automatically"""
        url = 'https://docs.google.com/document/d/1abc123/edit'
        doc_id = '1abc123'
        doc_type = 'document'
        
        download_path, result_id, files = self.iadrive.download_google_doc(url, doc_id, doc_type)
        
        self.assertEqual(result_id, doc_id)
        self.assertTrue(os.path.exists(download_path))
        # Should have all 7 document formats: pdf, docx, odt, rtf, txt, html, epub
        self.assertEqual(len(files), 7)
        
        # Check that all expected formats are present
        extensions = [os.path.splitext(f)[1].lower() for f in files]
        expected_extensions = ['.pdf', '.docx', '.odt', '.rtf', '.txt', '.html', '.epub']
        for ext in expected_extensions:
            self.assertIn(ext, extensions, f"Missing format: {ext}")
        
        # Check that files were created and have content
        for file_path in files:
            self.assertTrue(os.path.exists(file_path))
            self.assertGreater(os.path.getsize(file_path), 0)

    @patch('iadrive.core.urllib.request', MockUrllibRequest)
    def test_download_google_sheet_all_formats(self):
        """Test downloading Google Spreadsheet with all formats automatically"""
        url = 'https://docs.google.com/spreadsheets/d/1abc123/edit'
        doc_id = '1abc123'
        doc_type = 'spreadsheets'
        
        download_path, result_id, files = self.iadrive.download_google_doc(url, doc_id, doc_type)
        
        self.assertEqual(result_id, doc_id)
        # Should have all 6 spreadsheet formats: xlsx, ods, pdf, csv, tsv, html
        self.assertEqual(len(files), 6)
        
        # Check that all expected formats are present
        extensions = [os.path.splitext(f)[1].lower() for f in files]
        expected_extensions = ['.xlsx', '.ods', '.pdf', '.csv', '.tsv', '.html']
        for ext in expected_extensions:
            self.assertIn(ext, extensions, f"Missing format: {ext}")

    @patch('iadrive.core.urllib.request', MockUrllibRequest)
    def test_download_google_presentation_all_formats(self):
        """Test downloading Google Presentation with all formats automatically"""
        url = 'https://docs.google.com/presentation/d/1abc123/edit'
        doc_id = '1abc123'
        doc_type = 'presentation'
        
        download_path, result_id, files = self.iadrive.download_google_doc(url, doc_id, doc_type)
        
        self.assertEqual(result_id, doc_id)
        # Should have all 7 presentation formats: pdf, pptx, odp, txt, jpeg, png, svg
        self.assertEqual(len(files), 7)
        
        # Check that all expected formats are present
        extensions = [os.path.splitext(f)[1].lower() for f in files]
        expected_extensions = ['.pdf', '.pptx', '.odp', '.txt', '.jpeg', '.png', '.svg']
        for ext in expected_extensions:
            self.assertIn(ext, extensions, f"Missing format: {ext}")

    @patch('iadrive.core.urllib.request', MockUrllibRequest)
    def test_get_google_docs_title(self):
        """Test getting Google Docs title"""
        test_cases = [
            ('https://docs.google.com/document/d/1abc123/edit', 'Test Document'),
            ('https://docs.google.com/spreadsheets/d/1abc123/edit', 'Test Spreadsheet'),
            ('https://docs.google.com/presentation/d/1abc123/edit', 'Test Presentation')
        ]
        
        for url, expected in test_cases:
            result = self.iadrive.get_google_docs_title(url, '1abc123')
            self.assertEqual(result, expected)

    def test_create_metadata_google_docs(self):
        """Test metadata creation for Google Docs"""
        # Create test files
        test_doc_path = os.path.join(self.test_dir, 'Test Document.pdf')
        with open(test_doc_path, 'w') as f:
            f.write('test pdf content')
        
        file_map = {'Test Document.pdf': test_doc_path}
        drive_id = 'test123'
        url = 'https://docs.google.com/document/d/test123/edit'
        
        metadata = self.iadrive.create_metadata(
            file_map, drive_id, url, None, is_google_docs=True, doc_type='document'
        )
        
        self.assertEqual(metadata['title'], 'Test Document')
        self.assertEqual(metadata['mediatype'], 'texts')
        self.assertEqual(metadata['collection'], 'opensource_media')
        self.assertEqual(metadata['doctype'], 'document')
        self.assertIn('document', metadata['subject'])
        self.assertIn('Google Document exported', metadata['description'])

    @patch('iadrive.core.internetarchive')
    def test_upload_to_ia_google_docs(self, mock_ia):
        """Test uploading Google Docs to Internet Archive"""
        # Mock IA item
        mock_item = MagicMock()
        mock_item.exists = False
        mock_ia.get_item.return_value = mock_item
        
        # Create test file
        test_file = os.path.join(self.test_dir, 'test.pdf')
        with open(test_file, 'w') as f:
            f.write('test')
        
        file_map = {'test.pdf': test_file}
        drive_id = 'test123'
        metadata = {'title': 'Test Doc', 'mediatype': 'texts'}
        
        identifier, result_metadata = self.iadrive.upload_to_ia(
            file_map, drive_id, metadata, is_google_docs=True
        )
        
        self.assertEqual(identifier, 'docs-test123')  # docs- prefix for Google Docs
        self.assertEqual(result_metadata, metadata)
        mock_item.upload.assert_called_once()

    @patch('iadrive.core.gdown', MockGdown)
    def test_download_drive_content_file(self):
        """Test downloading a single file (original functionality)"""
        url = 'https://drive.google.com/file/d/1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms/edit'
        
        download_path, drive_id, files = self.iadrive.download_drive_content(url)
        
        self.assertEqual(drive_id, '1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms')
        self.assertTrue(os.path.exists(download_path))
        self.assertIsNone(files)  # Regular drive downloads don't return file list

    @patch('iadrive.core.urllib.request', MockUrllibRequest)
    def test_download_drive_content_google_docs(self):
        """Test downloading Google Docs"""
        url = 'https://docs.google.com/document/d/1abc123/edit'
        
        download_path, drive_id, files = self.iadrive.download_drive_content(url)
        
        self.assertEqual(drive_id, '1abc123')
        self.assertTrue(os.path.exists(download_path))
        self.assertIsNotNone(files)  # Google Docs downloads return file list
        self.assertGreater(len(files), 0)

    def test_docs_formats_structure(self):
        """Test that docs_formats structure is correct"""
        # Check that all document types are present
        self.assertIn('document', self.iadrive.docs_formats)
        self.assertIn('spreadsheets', self.iadrive.docs_formats)
        self.assertIn('presentation', self.iadrive.docs_formats)
        
        # Check some expected formats
        self.assertIn('pdf', self.iadrive.docs_formats['document'])
        self.assertIn('xlsx', self.iadrive.docs_formats['spreadsheets'])
        self.assertIn('pptx', self.iadrive.docs_formats['presentation'])
        
        # Ensure we have all the expected formats for comprehensive export
        doc_formats = self.iadrive.docs_formats['document']
        self.assertEqual(len(doc_formats), 7)  # pdf, docx, odt, rtf, txt, html, epub
        
        sheet_formats = self.iadrive.docs_formats['spreadsheets']
        self.assertEqual(len(sheet_formats), 6)  # xlsx, ods, pdf, csv, tsv, html
        
        pres_formats = self.iadrive.docs_formats['presentation']
        self.assertEqual(len(pres_formats), 7)  # pdf, pptx, odp, txt, jpeg, png, svg


class IAdriveIntegrationMockTest(unittest.TestCase):
    """Integration tests with mocked external dependencies"""

    def setUp(self):
        self.test_dir = tempfile.mkdtemp()
        self.iadrive = IAdrive(verbose=False, dir_path=self.test_dir)

    def tearDown(self):
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    @patch('iadrive.core.gdown', MockGdown)
    @patch('iadrive.core.internetarchive')
    def test_archive_drive_url_full_workflow_traditional(self, mock_ia):
        """Test the complete workflow for traditional Google Drive content"""
        # Mock IA session and item
        mock_session = MagicMock()
        mock_session.config = {'s3': {'access': 'test_key'}}
        mock_ia.get_session.return_value = mock_session
        
        mock_item = MagicMock()
        mock_item.exists = False
        mock_ia.get_item.return_value = mock_item
        
        url = 'https://drive.google.com/drive/folders/1-0axLqCuOUNbBIe3Cz6Y1KojGg4iXg1h'
        
        identifier, metadata = self.iadrive.archive_drive_url(url)
        
        # Updated expectation, preserve original case in drive ID
        self.assertEqual(identifier, 'drive-1-0axLqCuOUNbBIe3Cz6Y1KojGg4iXg1h')
        self.assertEqual(metadata['title'], 'Test Folder')
        self.assertEqual(metadata['originalurl'], url)
        self.assertEqual(metadata['mediatype'], 'data')
        mock_item.upload.assert_called_once()

    @patch('iadrive.core.urllib.request', MockUrllibRequest)
    @patch('iadrive.core.internetarchive')
    def test_archive_drive_url_full_workflow_google_docs(self, mock_ia):
        """Test the complete workflow for Google Docs"""
        # Mock IA session and item
        mock_session = MagicMock()
        mock_session.config = {'s3': {'access': 'test_key'}}
        mock_ia.get_session.return_value = mock_session
        
        mock_item = MagicMock()
        mock_item.exists = False
        mock_ia.get_item.return_value = mock_item
        
        url = 'https://docs.google.com/document/d/1abc123/edit'
        
        identifier, metadata = self.iadrive.archive_drive_url(url)
        
        self.assertEqual(identifier, 'docs-1abc123')
        self.assertEqual(metadata['title'], 'Test Document')
        self.assertEqual(metadata['originalurl'], url)
        self.assertEqual(metadata['mediatype'], 'texts')
        self.assertEqual(metadata['doctype'], 'document')
        self.assertEqual(metadata['filecount'], '7')
        mock_item.upload.assert_called_once()


if __name__ == '__main__':
    unittest.main()


import unittest
import os
import tempfile
import time
from datetime import datetime

from iadrive.utils import (
    key_value_to_dict, 
    sanitize_identifier, 
    get_oldest_file_date, 
    extract_file_types,
    get_collaborators
)


class UtilsTest(unittest.TestCase):

    def test_key_value_to_dict_single_item(self):
        """Test converting single key:value to dict"""
        result = key_value_to_dict(['key:value'])
        expected = {'key': 'value'}
        self.assertEqual(result, expected)

    def test_key_value_to_dict_multiple_items(self):
        """Test converting multiple key:value pairs to dict"""
        result = key_value_to_dict(['key1:value1', 'key2:value2'])
        expected = {'key1': 'value1', 'key2': 'value2'}
        self.assertEqual(result, expected)

    def test_key_value_to_dict_multiple_values_same_key(self):
        """Test multiple values for same key"""
        result = key_value_to_dict(['key:value1', 'key:value2'])
        expected = {'key': ['value1', 'value2']}
        self.assertEqual(result, expected)

    def test_key_value_to_dict_empty_list(self):
        """Test empty input"""
        result = key_value_to_dict([])
        expected = {}
        self.assertEqual(result, expected)

    def test_key_value_to_dict_none_input(self):
        """Test None input"""
        result = key_value_to_dict(None)
        expected = {}
        self.assertEqual(result, expected)

    def test_sanitize_identifier_valid(self):
        """Test preserving valid identifiers"""
        valid = [
            'drive-1-0axLqCuOUNbBIe3Cz6Y1KojGg4iXg1h',
            'drive-test123',
            'drive-abc_def-ghi'
        ]
        clean = [sanitize_identifier(x) for x in valid]
        self.assertEqual(valid, clean)

    def test_sanitize_identifier_invalid_chars(self):
        """Test sanitizing invalid characters"""
        bad = [
            'drive:test:123',
            'drive@test#123',
            'drive test 123'
        ]
        expected = [
            'drive-test-123',
            'drive-test-123',
            'drive-test-123'
        ]
        clean = [sanitize_identifier(x) for x in bad]
        self.assertEqual(expected, clean)

    def test_sanitize_identifier_case_preservation(self):
        """Test preserving original case"""
        result = sanitize_identifier('DRIVE-Test123')
        expected = 'DRIVE-Test123'  # Should preserve original case
        self.assertEqual(result, expected)

    def test_sanitize_identifier_consecutive_hyphens(self):
        """Test removing consecutive hyphens"""
        result = sanitize_identifier('drive---test---123')
        expected = 'drive-test-123'
        self.assertEqual(result, expected)

    def test_sanitize_identifier_leading_trailing_hyphens(self):
        """Test removing leading/trailing hyphens"""
        result = sanitize_identifier('-drive-test-123-')
        expected = 'drive-test-123'
        self.assertEqual(result, expected)

    def test_get_oldest_file_date(self):
        """Test getting oldest file date"""
        with tempfile.TemporaryDirectory() as temp_dir:
            # Create files with different timestamps
            file1 = os.path.join(temp_dir, 'file1.txt')
            file2 = os.path.join(temp_dir, 'file2.txt')
            
            with open(file1, 'w') as f:
                f.write('content1')
            with open(file2, 'w') as f:
                f.write('content2')
            
            # Modify file1 to be older
            old_time = time.time() - 86400  # 1 day ago
            os.utime(file1, (old_time, old_time))
            
            files = [file1, file2]
            date_str, year_str = get_oldest_file_date(files)
            
            expected_date = datetime.fromtimestamp(old_time).strftime('%Y-%m-%d')
            expected_year = datetime.fromtimestamp(old_time).strftime('%Y')
            
            self.assertEqual(date_str, expected_date)
            self.assertEqual(year_str, expected_year)

    def test_get_oldest_file_date_no_files(self):
        """Test getting oldest file date with empty list"""
        files = []
        date_str, year_str = get_oldest_file_date(files)
        
        # Should return current date as fallback
        current_date = datetime.now().strftime('%Y-%m-%d')
        current_year = datetime.now().strftime('%Y')
        
        self.assertEqual(date_str, current_date)
        self.assertEqual(year_str, current_year)

    def test_extract_file_types(self):
        """Test extracting file types from file list"""
        files = [
            '/path/to/file.txt',
            '/path/to/file.PDF',
            '/path/to/file.docx',
            '/path/to/another.txt',
            '/path/to/noextension'
        ]
        
        result = extract_file_types(files)
        expected = ['docx', 'pdf', 'txt']  # sorted, lowercase, unique
        
        self.assertEqual(result, expected)

    def test_extract_file_types_empty_list(self):
        """Test extracting file types from empty list"""
        result = extract_file_types([])
        expected = []
        self.assertEqual(result, expected)

    def test_get_collaborators(self):
        """Test get_collaborators function"""
        # This function currently returns None as it needs Google Drive API
        result = get_collaborators('test123')
        self.assertIsNone(result)


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""IAdrive - Download Google Drive files/folders and Google Docs and upload to Internet Archive

Usage:
  iadrive <url> [--metadata=<key:value>...] [--disable-slash-files] [--quiet] [--debug]
  iadrive -h | --help
  iadrive --version

Arguments:
  <url>                         Google Drive URL (file or folder) or Google Docs URL

Options:
  -h --help                    Show this screen
  --metadata=<key:value>       Custom metadata to add to the archive.org item
  --disable-slash-files        Upload files without preserving folder structure
  -q --quiet                   Just print errors
  -d --debug                   Print all logs to stdout

Note: Google Docs are automatically exported in all available formats for preservation:
  Documents: PDF, DOCX, ODT, RTF, TXT, HTML, EPUB
  Spreadsheets: XLSX, ODS, PDF, CSV, TSV, HTML
  Presentations: PDF, PPTX, ODP, TXT, JPEG, PNG, SVG
"""

import sys
import docopt
import logging
import traceback

from iadrive.core import IAdrive
from iadrive.utils import key_value_to_dict, get_latest_pypi_version
from iadrive import __version__


def print_supported_formats():
    """Print supported formats for Google Docs"""
    print("\nGoogle Docs formats (ALL are automatically exported):")
    print("  Documents: PDF, DOCX, ODT, RTF, TXT, HTML, EPUB")
    print("  Spreadsheets: XLSX, ODS, PDF, CSV, TSV, HTML") 
    print("  Presentations: PDF, PPTX, ODP, TXT, JPEG, PNG, SVG")
    print("\nFor maximum preservation, IAdrive exports ALL available formats automatically.")


def main():
    args = docopt.docopt(__doc__, version=__version__)
    
    url = args['<url>']
    quiet_mode = args['--quiet']
    debug_mode = args['--debug']
    disable_slash_files = args['--disable-slash-files']
    
    if debug_mode:
        root = logging.getLogger()
        root.setLevel(logging.DEBUG)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '\033[92m[DEBUG]\033[0m %(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        root.addHandler(ch)
    
    metadata = key_value_to_dict(args['--metadata'])
    
    # Check if this is a Google Docs URL and provide info
    is_docs_url = any(pattern in url for pattern in ['docs.google.com/document', 'docs.google.com/spreadsheets', 'docs.google.com/presentation'])
    
    if is_docs_url and not quiet_mode:
        print("Detected Google Docs URL.")
        print("All available formats will be automatically exported for comprehensive preservation.")
        print_supported_formats()
        print()
    
    iadrive = IAdrive(verbose=not quiet_mode, preserve_folders=not disable_slash_files)
    
    try:
        identifier, meta = iadrive.archive_drive_url(url, metadata)
        print('\n:: Upload Finished. Item information:')
        print('Title: %s' % meta['title'])
        print('Item URL: https://archive.org/details/%s\n' % identifier)
        
        # Show what was uploaded for Google Docs
        if is_docs_url and not quiet_mode:
            doc_type = None
            if 'docs.google.com/document' in url:
                doc_type = 'document'
            elif 'docs.google.com/spreadsheets' in url:
                doc_type = 'spreadsheets'  
            elif 'docs.google.com/presentation' in url:
                doc_type = 'presentation'
                
            if doc_type:
                format_count = len([ext for ext in ['pdf', 'docx', 'odt', 'rtf', 'txt', 'html', 'epub', 'xlsx', 'ods', 'csv', 'tsv', 'pptx', 'odp', 'jpeg', 'png', 'svg']])
                print(f"Google {doc_type.title()} was exported and uploaded in ALL available formats.")
                print("Check the Internet Archive item for complete format preservation.\n")
        
    except Exception as e:
        error_msg = str(e)
        print('\n\033[91m'
              f'An exception occurred: {error_msg}\n')
              
        print('If this isn\'t a connection problem, please report to '
              'https://github.com/Andres9890/iadrive/issues')
        
        if debug_mode:
            traceback.print_exc()
        print('\033[0m')
        sys.exit(1)
    finally:
        # Version check after upload attempt (success or fail)
        latest_version = get_latest_pypi_version()
        if latest_version and latest_version != __version__:
            print(f"\033[93mA newer version of IAdrive is available: \033[92m{latest_version}\033[0m")
            print("Update with: pip install --upgrade iadrive\n")


if __name__ == '__main__':
    main()


import os
import re
import glob
import time
import logging
import subprocess
import internetarchive
import urllib.request
import urllib.error
from datetime import datetime
from pathlib import Path
from urllib.parse import urlparse, parse_qs

from iadrive.utils import sanitize_identifier, get_oldest_file_date, extract_file_types, get_collaborators
from iadrive import __version__


class IAdrive:
    def __init__(self, verbose=False, dir_path='~/.iadrive', preserve_folders=True):
        """
        IAdrive - Google Drive to Internet Archive uploader
        
        :param verbose: Print detailed logs
        :param dir_path: Directory to store downloaded files
        :param preserve_folders: Whether to preserve folder structure in uploaded files
        """
        self.verbose = verbose
        self.preserve_folders = preserve_folders
        self.logger = logging.getLogger(__name__)
        self.dir_path = os.path.expanduser(dir_path)
        
        # Create download directory
        os.makedirs(self.dir_path, exist_ok=True)
        
        if not verbose:
            self.logger.setLevel(logging.ERROR)
        
        # Google Docs export formats
        self.docs_formats = {
            'document': {
                'pdf': 'PDF',
                'docx': 'Microsoft Word',
                'odt': 'OpenDocument Text',
                'rtf': 'Rich Text Format',
                'txt': 'Plain Text',
                'html': 'HTML',
                'epub': 'EPUB'
            },
            'spreadsheets': {
                'xlsx': 'Microsoft Excel',
                'ods': 'OpenDocument Spreadsheet',
                'pdf': 'PDF',
                'csv': 'CSV (first sheet)',
                'tsv': 'TSV (first sheet)',
                'html': 'HTML'
            },
            'presentation': {
                'pdf': 'PDF',
                'pptx': 'Microsoft PowerPoint',
                'odp': 'OpenDocument Presentation',
                'txt': 'Plain Text',
                'jpeg': 'JPEG (slides as images)',
                'png': 'PNG (slides as images)',
                'svg': 'SVG (slides as images)'
            }
        }
        
    
    def check_dependencies(self):
        """Check if required dependencies are installed and configured"""
        try:
            import gdown
            import internetarchive
        except ImportError as e:
            raise Exception(f"Missing required package: {e}. Run 'pip install -r requirements.txt'")
        
        # Check if internetarchive is configured
        try:
            ia_config = internetarchive.get_session().config
            if not ia_config.get('s3', {}).get('access'):
                raise Exception("Internet Archive not configured. Run 'ia configure' first.")
        except Exception as e:
            raise Exception(f"Internet Archive configuration error: {e}")
    
    def is_google_docs_url(self, url):
        """Check if URL is a Google Docs/Sheets/Slides URL"""
        docs_patterns = [
            r'docs\.google\.com/document',
            r'docs\.google\.com/spreadsheets',
            r'docs\.google\.com/presentation'
        ]
        return any(re.search(pattern, url) for pattern in docs_patterns)
    
    def get_google_docs_type(self, url):
        """Determine the type of Google Docs URL"""
        if 'docs.google.com/document' in url:
            return 'document'
        elif 'docs.google.com/spreadsheets' in url:
            return 'spreadsheets'
        elif 'docs.google.com/presentation' in url:
            return 'presentation'
        else:
            return None
    
    def extract_docs_id(self, url):
        """Extract Google Docs ID from URL"""
        patterns = [
            r'/document/d/([a-zA-Z0-9-_]+)',
            r'/spreadsheets/d/([a-zA-Z0-9-_]+)',
            r'/presentation/d/([a-zA-Z0-9-_]+)',
        ]
        
        for pattern in patterns:
            match = re.search(pattern, url)
            if match:
                return match.group(1)
        
        raise ValueError(f"Could not extract Google Docs ID from URL: {url}")
    
    def extract_drive_id(self, url):
        """Extract Google Drive file/folder ID from URL"""
        if self.is_google_docs_url(url):
            return self.extract_docs_id(url)
        
        patterns = [
            r'/folders/([a-zA-Z0-9-_]+)',
            r'/file/d/([a-zA-Z0-9-_]+)',
            r'id=([a-zA-Z0-9-_]+)',
        ]
        
        for pattern in patterns:
            match = re.search(pattern, url)
            if match:
                return match.group(1)
        
        raise ValueError(f"Could not extract Google Drive ID from URL: {url}")
    
    def is_folder_url(self, url):
        """Check if URL is a Google Drive folder"""
        return '/folders/' in url
    
    def download_google_doc(self, url, doc_id, doc_type, formats=None):
        """Download Google Doc in all available formats for preservation"""
        all_formats = list(self.docs_formats.get(doc_type, {}).keys())
        
        if not all_formats:
            raise Exception(f"No formats available for document type: {doc_type}")
        
        download_path = os.path.join(self.dir_path, f"docs-{doc_id}")
        os.makedirs(download_path, exist_ok=True)
        
        if self.verbose:
            print(f"Downloading Google {doc_type.title()} from: {url}")
            print(f"Exporting ALL {len(all_formats)} available formats: {', '.join(all_formats)}")
            print(f"Download path: {download_path}")
        
        downloaded_files = []
        
        # Get document title from the original URL (try to fetch HTML and extract title)
        doc_title = self.get_google_docs_title(url, doc_id)
        
        for fmt in all_formats:
            # All formats in docs_formats are valid by definition
            try:
                export_url = f"https://docs.google.com/{doc_type}/d/{doc_id}/export?format={fmt}"
                
                # Create filename
                safe_title = re.sub(r'[^\w\s-]', '', doc_title)[:50]
                filename = f"{safe_title}.{fmt}" if safe_title else f"{doc_type}_{doc_id}.{fmt}"
                file_path = os.path.join(download_path, filename)
                
                if self.verbose:
                    print(f"  Downloading {fmt.upper()}: {filename}")
                
                # Download the file
                urllib.request.urlretrieve(export_url, file_path)
                
                # Check if file was downloaded successfully and has content
                if os.path.exists(file_path) and os.path.getsize(file_path) > 0:
                    downloaded_files.append(file_path)
                    if self.verbose:
                        size_mb = os.path.getsize(file_path) / (1024 * 1024)
                        print(f"    Downloaded successfully ({size_mb:.2f} MB)")
                else:
                    if os.path.exists(file_path):
                        os.remove(file_path)
                    if self.verbose:
                        print(f"    Failed to download or empty file")
                        
            except Exception as e:
                if self.verbose:
                    print(f"  Error downloading {fmt}: {e}")
                continue
        
        if not downloaded_files:
            raise Exception(f"Failed to download Google {doc_type} in any format")
        
        return download_path, doc_id, downloaded_files
    
    def get_google_docs_title(self, url, doc_id):
        """Try to get title of Google Doc"""
        try:
            # Try to fetch the document page and extract title
            response = urllib.request.urlopen(url, timeout=10)
            html = response.read().decode('utf-8', errors='ignore')
            
            # Look for title in HTML
            title_match = re.search(r'<title>([^<]+)</title>', html, re.IGNORECASE)
            if title_match:
                title = title_match.group(1)
                # Remove " - Google Docs/Sheets/Slides" suffix
                title = re.sub(r' - Google (Docs|Sheets|Slides).*$', '', title)
                return title.strip()
        except:
            pass
        
        # Fallback to document ID
        return f"document_{doc_id}"
    
    def download_drive_content(self, url, formats=None):
        """Download Google Drive file/folder or Google Docs using gdown or direct export"""
        if self.is_google_docs_url(url):
            doc_id = self.extract_docs_id(url)
            doc_type = self.get_google_docs_type(url)
            return self.download_google_doc(url, doc_id, doc_type)  # No formats parameter needed
        else:
            # Handle regular Google Drive files/folders
            import gdown
            
            drive_id = self.extract_drive_id(url)
            download_path = os.path.join(self.dir_path, f"drive-{drive_id}")
            
            if self.verbose:
                print(f"Downloading from: {url}")
                print(f"Download path: {download_path}")
            
            try:
                if self.is_folder_url(url):
                    gdown.download_folder(url, output=download_path, quiet=not self.verbose)
                else:
                    os.makedirs(download_path, exist_ok=True)
                    gdown.download(url, output=download_path, quiet=not self.verbose, fuzzy=True)
                
                return download_path, drive_id, None
            except Exception as e:
                raise Exception(f"Failed to download from Google Drive: {e}")
    
    def get_file_list_with_structure(self, path):
        """
        Get list of all files with their relative paths preserved
        Returns a dictionary mapping relative paths to absolute paths
        """
        file_map = {}
        
        if os.path.isfile(path):
            # Single file case
            file_map[os.path.basename(path)] = path
        else:
            # Directory case - walk through and preserve structure
            for root, dirs, filenames in os.walk(path):
                for filename in filenames:
                    abs_path = os.path.join(root, filename)
                    # Get relative path from the download directory
                    rel_path = os.path.relpath(abs_path, path)
                    # Use forward slashes for consistency in archive
                    rel_path = rel_path.replace(os.sep, '/')
                    file_map[rel_path] = abs_path
        
        return file_map
    
    def get_file_list(self, path):
        """Get list of all files in the downloaded content (for backward compatibility)"""
        file_map = self.get_file_list_with_structure(path)
        return list(file_map.values())
    
    def create_metadata(self, file_map, drive_id, original_url, custom_meta=None, is_google_docs=False, doc_type=None):
        """Create Internet Archive metadata from downloaded files"""
        if not file_map:
            raise Exception("No files found to upload")
        
        files = list(file_map.values())
        
        # Get oldest file date
        oldest_date, oldest_year = get_oldest_file_date(files)
        
        # Determine title
        if is_google_docs:
            # For Google Docs, we try to get a title from the first file
            first_file = list(file_map.keys())[0]
            title = os.path.splitext(first_file)[0]  # Remove extension
            if title.startswith(f"{doc_type}_"):
                title = title.replace(f"{doc_type}_", "").replace(drive_id, "Document")
        elif len(files) == 1 and os.path.isfile(files[0]):
            # Single file
            title = os.path.basename(files[0])
        else:
            # Folder or multiple files
            # Try to get folder name from the first file's path
            common_path = os.path.commonpath(files) if len(files) > 1 else os.path.dirname(files[0])
            title = os.path.basename(common_path) or f"drive-{drive_id}"
        
        # Extract file types
        file_types = extract_file_types(files)
        
        # Get collaborators (this would need Google Drive API access in a real implementation)
        creator = get_collaborators(drive_id) or "IAdrive"
        
        # Create file listing for description with folder structure
        description_lines = []
        if is_google_docs:
            description_lines.append(f"Google {doc_type.title()} exported in:")
        else:
            description_lines.append("Files included:")
            
        for rel_path, abs_path in sorted(file_map.items()):
            file_size = os.path.getsize(abs_path)
            # Format size for readability
            if file_size > 1024 * 1024:
                size_str = f"{file_size / (1024 * 1024):.2f} MB"
            elif file_size > 1024:
                size_str = f"{file_size / 1024:.2f} KB"
            else:
                size_str = f"{file_size} bytes"
            
            if is_google_docs:
                # For Google Docs, show format information
                ext = os.path.splitext(rel_path)[1].lstrip('.')
                format_desc = self.docs_formats.get(doc_type, {}).get(ext, ext.upper())
                description_lines.append(f"- {rel_path} ({format_desc}, {size_str})")
            else:
                description_lines.append(f"- {rel_path} ({size_str})")
                
        description = "<br>".join(description_lines)
        
        # Create subject tags
        if is_google_docs:
            subject_tags = ["google", doc_type, "document"] + file_types
        else:
            subject_tags = ["google", "drive"] + file_types
        subject = ";".join(subject_tags) + ";"
        
        # Truncate subject if too long (IA limit is 255 bytes)
        while len(subject.encode('utf-8')) > 255:
            subject_tags.pop()
            subject = ";".join(subject_tags) + ";"
        
        # Add foldercount metadata (if any folders are present)
        folder_paths = set()
        for rel_path in file_map.keys():
            folder = os.path.dirname(rel_path)
            if folder and folder != '.':
                folder_paths.add(folder)
                
        # Set collection and mediatype based on content type
        if is_google_docs:
            collection = 'opensource_media'
            mediatype = 'texts' if doc_type in ['document'] else 'data'
        else:
            collection = 'opensource_media'
            mediatype = 'data'
                
        metadata = {
            'mediatype': mediatype,
            'collection': collection,
            'title': title,
            'description': description,
            'date': oldest_date,
            'year': oldest_year,
            'creator': creator,
            'subject': subject,
            'filecount': str(len(files)),
            'originalurl': original_url,
            'scanner': f'IAdrive File Mirroring Application {__version__}',
            **({'foldercount': str(len(folder_paths))} if folder_paths else {}),
            **({'doctype': doc_type} if is_google_docs else {})
        }
        
        if custom_meta:
            metadata.update(custom_meta)
        
        return metadata
    
    def upload_to_ia(self, file_map, drive_id, metadata, is_google_docs=False):
        """Upload files to Internet Archive with optional folder structure preservation"""
        if is_google_docs:
            identifier = f"docs-{drive_id}"
        else:
            identifier = f"drive-{drive_id}"
        identifier = sanitize_identifier(identifier)
        
        if self.verbose:
            print(f"Uploading to Internet Archive with identifier: {identifier}")
            if self.preserve_folders:
                print("Folder structure will be preserved")
            else:
                print("Files will be uploaded with flat structure (no folders)")
        
        item = internetarchive.get_item(identifier)
        
        # Check if item already exists
        if item.exists:
            if self.verbose:
                print(f"Item {identifier} already exists on archive.org")
            return identifier, metadata
        
        # Prepare files for upload
        upload_files = {}
        
        if self.preserve_folders:
            # Preserve folder structure
            for rel_path, abs_path in file_map.items():
                # Sanitize the relative path for IA (replace problematic characters)
                # But keep the folder structure with forward slashes
                safe_rel_path = rel_path.replace('\\', '/')
                # Remove any leading slash
                safe_rel_path = safe_rel_path.lstrip('/')
                upload_files[safe_rel_path] = abs_path
                
                if self.verbose:
                    print(f"  Preparing: {abs_path} -> {safe_rel_path}")
        else:
            # Flat structure - use only filenames, handle duplicates
            filename_counts = {}
            for rel_path, abs_path in file_map.items():
                filename = os.path.basename(abs_path)
                
                # Handle duplicate filenames by adding a counter
                if filename in filename_counts:
                    filename_counts[filename] += 1
                    name, ext = os.path.splitext(filename)
                    filename = f"{name}_{filename_counts[filename]}{ext}"
                else:
                    filename_counts[filename] = 0
                
                upload_files[filename] = abs_path
                
                if self.verbose:
                    print(f"  Preparing: {abs_path} -> {filename}")
        
        # Upload files
        try:
            if self.verbose:
                print(f"  Uploading {len(upload_files)} files...")
            
            response = item.upload(upload_files, metadata=metadata, 
                                 retries=3, verbose=self.verbose)
            
            if self.verbose:
                print(f"Successfully uploaded {len(upload_files)} files")
        except Exception as e:
            raise Exception(f"Failed to upload to Internet Archive: {e}")
        
        return identifier, metadata
    
    def archive_drive_url(self, url, custom_meta=None, formats=None):
        """Main method to download from Google Drive/Docs and upload to IA"""
        # Check dependencies first
        self.check_dependencies()
        
        is_google_docs = self.is_google_docs_url(url)
        doc_type = self.get_google_docs_type(url) if is_google_docs else None
        
        # For Google Docs, we always export all formats
        if is_google_docs:
            download_path, drive_id, downloaded_files = self.download_drive_content(url)
        else:
            download_path, drive_id, _ = self.download_drive_content(url)
        
        # Get file list with structure preserved
        file_map = self.get_file_list_with_structure(download_path)
        if not file_map:
            raise Exception("No files downloaded")
        
        if self.verbose:
            print(f"Found {len(file_map)} files to upload")
            if is_google_docs:
                print(f"Google {doc_type.title()} exported in ALL available formats:")
                for rel_path in sorted(file_map.keys()):
                    ext = os.path.splitext(rel_path)[1].lstrip('.')
                    format_desc = self.docs_formats.get(doc_type, {}).get(ext, ext.upper())
                    print(f"  - {rel_path} ({format_desc})")
            else:
                if self.preserve_folders:
                    print("File structure:")
                    for rel_path in sorted(file_map.keys()):
                        print(f"  - {rel_path}")
                else:
                    print("Files will be uploaded with flat structure")
        
        # Create metadata
        metadata = self.create_metadata(file_map, drive_id, url, custom_meta, is_google_docs, doc_type)
        
        # Upload to Internet Archive with or without folder structure
        identifier, final_metadata = self.upload_to_ia(file_map, drive_id, metadata, is_google_docs)
        
        # Clean up downloaded files
        import shutil
        shutil.rmtree(download_path)
        if self.verbose:
            print("Cleaned up temporary files")
        
        return identifier, final_metadata


import os
import re
import time
from datetime import datetime
from pathlib import Path
from collections import defaultdict


def key_value_to_dict(lst):
    """Convert key:value pair strings into a dictionary"""
    if not lst:
        return {}
    
    if not isinstance(lst, list):
        lst = [lst]
    
    result = defaultdict(list)
    for item in lst:
        if ':' not in item:
            continue
        key, value = item.split(':', 1)
        if value and value not in result[key]:
            result[key].append(value)
        elif not result[key]:
            result[key] = [value]
    
    return {k: v if len(v) > 1 else v[0] for k, v in result.items()}


def sanitize_identifier(identifier, replacement='-'):
    """Sanitize identifier for Internet Archive"""
    # IA identifiers must be alphanumeric with hyphens/underscores
    # Remove the lowercase conversion to preserve original case from Drive/Docs IDs
    identifier = re.sub(r'[^\w-]', replacement, identifier)
    # Remove consecutive hyphens
    identifier = re.sub(r'-+', '-', identifier)
    # Remove leading/trailing hyphens
    identifier = identifier.strip('-')
    return identifier


def get_oldest_file_date(files):
    """Get the oldest file modification date"""
    oldest_timestamp = float('inf')
    
    for file_path in files:
        if os.path.exists(file_path):
            mtime = os.path.getmtime(file_path)
            if mtime < oldest_timestamp:
                oldest_timestamp = mtime
    
    if oldest_timestamp == float('inf'):
        # Fallback to current date
        oldest_timestamp = time.time()
    
    dt = datetime.fromtimestamp(oldest_timestamp)
    return dt.strftime('%Y-%m-%d'), dt.strftime('%Y')


def extract_file_types(files):
    """Extract unique file extensions from file list"""
    extensions = set()
    for file_path in files:
        ext = Path(file_path).suffix.lower().lstrip('.')
        if ext:
            extensions.add(ext)
    return sorted(list(extensions))


def get_collaborators(drive_id):
    """
    Get collaborators for a Google Drive file/folder
    Note: This would require Google Drive API access in a real implementation
    For now, returns None to fall back to "IAdrive"
    """
    # TODO: Implement Google Drive API integration to get collaborators
    # This would require OAuth2 setup and drive API calls
    return None


def get_latest_pypi_version(package_name="iadrive"):
    """
    Request PyPI for the latest version
    Returns the version string, or None if it cannot be determined
    """
    import json
    import urllib.request
    try:
        url = f"https://pypi.org/pypi/{package_name}/json"
        with urllib.request.urlopen(url, timeout=5) as response:
            data = json.load(response)
            return data["info"]["version"]
    except Exception:
        return None


from setuptools import setup, find_packages

# Read version from file instead of importing
def get_version():
    with open("iadrive/__init__.py", "r") as f:
        for line in f:
            if line.startswith("__version__"):
                return line.split("=")[1].strip().strip('"').strip("'")
    raise RuntimeError("Unable to find version string.")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="iadrive",
    version=get_version(),
    author="Andres99",
    description="Download Google Drive files/folders and upload them to the Internet Archive",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Andres9890/iadrive",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.9",
    install_requires=[
        "internetarchive>=5.5.0",
        "gdown>=5.2.0",
        "docopt-ng>=0.9.0",
        "python-dateutil>=2.9.0.post0",
    ],
    entry_points={
        "console_scripts": [
            "iadrive=iadrive.__main__:main",
        ],
    },
)